@@ -1,24 +1,28 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
+#if __TBB_TEST_USE_WSUGGEST_OVERRIDE
+// __TBB_override may not be used in the tested header file
+// #pragma GCC diagnostic ignored "-Wsuggest-override"
+#undef __TBB_TEST_USE_WSUGGEST_OVERRIDE
+#endif
+
 #include "harness_defs.h" // for suppress_unused_warning
+
+#if TBB_USE_EXCEPTIONS
 #include "harness_assert.h"
 #include "../../examples/common/utility/utility.h"
 #include <sstream>
@@ -572,9 +576,11 @@ void run_high_level_api_tests(){
     thread_range_tests::post_process_troika_value_with_ladder_step();
     thread_range_tests::test_print_content();
 }
+#endif // TBB_USE_EXCEPTIONS
 
 #include "harness.h"
 int TestMain(){
+#if TBB_USE_EXCEPTIONS
     Harness::suppress_unused_warning(utility::thread_number_range_desc);
     try{
         run_implementation_unit_tests();
@@ -585,5 +591,8 @@ int TestMain(){
         ASSERT(false,str.str().c_str());
     }
     return Harness::Done;
-
+#else
+    REPORT("Known issue: the test cannot work with exceptions disabled\n");
+    return Harness::Done;
+#endif
 }
