@@ -1,25 +1,26 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
+#if __TBB_CPF_BUILD
+#define TBB_DEPRECATED_FLOW_NODE_EXTRACTION 1
+#endif
+
 #include "harness_graph.h"
 
+#include "tbb/flow_graph.h"
 #include "tbb/task_scheduler_init.h"
 #include "tbb/spin_rw_mutex.h"
 
@@ -27,7 +28,7 @@
 #define MAX_NODES 4
 
 //! Performs test on function nodes with limited concurrency and buffering
-/** Theses tests check:
+/** These tests check:
     1) that the number of executing copies never exceed the concurrency limit
     2) that the node never rejects
     3) that no items are lost
@@ -53,12 +54,12 @@ template<typename IO>
 struct pass_through {
     IO operator()(const IO& i) { return i; }
 };
-     
+
 template< typename InputType, typename OutputType, typename Body >
 void buffered_levels( size_t concurrency, Body body ) {
 
    // Do for lc = 1 to concurrency level
-   for ( size_t lc = 1; lc <= concurrency; ++lc ) { 
+   for ( size_t lc = 1; lc <= concurrency; ++lc ) {
    tbb::flow::graph g;
 
    // Set the execute_counter back to zero in the harness
@@ -71,7 +72,7 @@ void buffered_levels( size_t concurrency, Body body ) {
    // Create the function_node with the appropriate concurrency level, and use default buffering
    tbb::flow::function_node< InputType, OutputType > exe_node( g, lc, body );
    tbb::flow::function_node<InputType, InputType> pass_thru( g, tbb::flow::unlimited, pass_through<InputType>());
-   
+
    // Create a vector of identical exe_nodes and pass_thrus
    std::vector< tbb::flow::function_node< InputType, OutputType > > exe_vec(2, exe_node);
    std::vector< tbb::flow::function_node< InputType, InputType > > pass_thru_vec(2, pass_thru);
@@ -85,9 +86,13 @@ void buffered_levels( size_t concurrency, Body body ) {
    // For num_receivers = 1 to MAX_NODES
    for (size_t num_receivers = 1; num_receivers <= MAX_NODES; ++num_receivers ) {
         // Create num_receivers counting receivers and connect the exe_vec[node_idx] to them.
-        harness_mapped_receiver<OutputType> *receivers = new harness_mapped_receiver<OutputType>[num_receivers];
+        std::vector< harness_mapped_receiver<OutputType>* > receivers(num_receivers);
+        for (size_t i = 0; i < num_receivers; i++) {
+            receivers[i] = new harness_mapped_receiver<OutputType>(g);
+        }
+
         for (size_t r = 0; r < num_receivers; ++r ) {
-            tbb::flow::make_edge( exe_vec[node_idx], receivers[r] );
+            tbb::flow::make_edge( exe_vec[node_idx], *receivers[r] );
         }
 
         // Do the test with varying numbers of senders
@@ -102,35 +107,39 @@ void buffered_levels( size_t concurrency, Body body ) {
 
             // Initialize the receivers so they know how many senders and messages to check for
             for (size_t r = 0; r < num_receivers; ++r ) {
-                 receivers[r].initialize_map( N, num_senders ); 
+                 receivers[r]->initialize_map( N, num_senders );
             }
 
             // Do the test
             NativeParallelFor( (int)num_senders, parallel_put_until_limit<InputType>(senders) );
             g.wait_for_all();
 
-            // confirm that each sender was requested from N times 
+            // confirm that each sender was requested from N times
             for (size_t s = 0; s < num_senders; ++s ) {
                 size_t n = senders[s].my_received;
-                ASSERT( n == N, NULL ); 
+                ASSERT( n == N, NULL );
                 ASSERT( senders[s].my_receiver == &pass_thru_vec[node_idx], NULL );
             }
             // validate the receivers
             for (size_t r = 0; r < num_receivers; ++r ) {
-                receivers[r].validate();
+                receivers[r]->validate();
             }
             delete [] senders;
         }
         for (size_t r = 0; r < num_receivers; ++r ) {
-            tbb::flow::remove_edge( exe_vec[node_idx], receivers[r] );
+            tbb::flow::remove_edge( exe_vec[node_idx], *receivers[r] );
         }
         ASSERT( exe_vec[node_idx].try_put( InputType() ) == true, NULL );
         g.wait_for_all();
         for (size_t r = 0; r < num_receivers; ++r ) {
             // since it's detached, nothing should have changed
-            receivers[r].validate();
+            receivers[r]->validate();
         }
-        delete [] receivers;
+
+        for (size_t i = 0; i < num_receivers; i++) {
+            delete receivers[i];
+        }
+
     } // for num_receivers
     } // for node_idx
     } // for concurrency level lc
@@ -149,7 +158,7 @@ struct inc_functor {
     int operator()( int i ) {
        ++global_execute_count;
        ++local_execute_count;
-       return i; 
+       return i;
     }
 
 };
@@ -158,19 +167,24 @@ template< typename InputType, typename OutputType >
 void buffered_levels_with_copy( size_t concurrency ) {
 
     // Do for lc = 1 to concurrency level
-    for ( size_t lc = 1; lc <= concurrency; ++lc ) { 
+    for ( size_t lc = 1; lc <= concurrency; ++lc ) {
         tbb::flow::graph g;
 
         inc_functor cf;
         cf.local_execute_count = Offset;
         global_execute_count = Offset;
-       
+
         tbb::flow::function_node< InputType, OutputType > exe_node( g, lc, cf );
 
         for (size_t num_receivers = 1; num_receivers <= MAX_NODES; ++num_receivers ) {
-           harness_mapped_receiver<OutputType> *receivers = new harness_mapped_receiver<OutputType>[num_receivers];
+
+           std::vector< harness_mapped_receiver<OutputType>* > receivers(num_receivers);
+           for (size_t i = 0; i < num_receivers; i++) {
+               receivers[i] = new harness_mapped_receiver<OutputType>(g);
+           }
+
            for (size_t r = 0; r < num_receivers; ++r ) {
-               tbb::flow::make_edge( exe_node, receivers[r] );
+               tbb::flow::make_edge( exe_node, *receivers[r] );
             }
 
             harness_counting_sender<InputType> *senders = NULL;
@@ -182,7 +196,7 @@ void buffered_levels_with_copy( size_t concurrency ) {
                 }
 
                 for (size_t r = 0; r < num_receivers; ++r ) {
-                    receivers[r].initialize_map( N, num_senders ); 
+                    receivers[r]->initialize_map( N, num_senders );
                 }
 
                 NativeParallelFor( (int)num_senders, parallel_put_until_limit<InputType>(senders) );
@@ -190,43 +204,44 @@ void buffered_levels_with_copy( size_t concurrency ) {
 
                 for (size_t s = 0; s < num_senders; ++s ) {
                     size_t n = senders[s].my_received;
-                    ASSERT( n == N, NULL ); 
+                    ASSERT( n == N, NULL );
                     ASSERT( senders[s].my_receiver == &exe_node, NULL );
                 }
                 for (size_t r = 0; r < num_receivers; ++r ) {
-                    receivers[r].validate();
+                    receivers[r]->validate();
                 }
                 delete [] senders;
             }
             for (size_t r = 0; r < num_receivers; ++r ) {
-                tbb::flow::remove_edge( exe_node, receivers[r] );
+                tbb::flow::remove_edge( exe_node, *receivers[r] );
             }
             ASSERT( exe_node.try_put( InputType() ) == true, NULL );
             g.wait_for_all();
             for (size_t r = 0; r < num_receivers; ++r ) {
-                receivers[r].validate();
+                receivers[r]->validate();
+            }
+
+            for (size_t i = 0; i < num_receivers; i++) {
+                delete receivers[i];
             }
-            delete [] receivers;
         }
 
         // validate that the local body matches the global execute_count and both are correct
         inc_functor body_copy = tbb::flow::copy_body<inc_functor>( exe_node );
-        const size_t expected_count = N/2 * MAX_NODES * MAX_NODES * ( MAX_NODES + 1 ) + MAX_NODES + Offset; 
+        const size_t expected_count = N/2 * MAX_NODES * MAX_NODES * ( MAX_NODES + 1 ) + MAX_NODES + Offset;
         size_t global_count = global_execute_count;
         size_t inc_count = body_copy.local_execute_count;
-        ASSERT( global_count == expected_count && global_count == inc_count, NULL ); 
-#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+        ASSERT( global_count == expected_count && global_count == inc_count, NULL );
         g.reset(tbb::flow::rf_reset_bodies);
         body_copy = tbb::flow::copy_body<inc_functor>( exe_node );
         inc_count = body_copy.local_execute_count;
-        ASSERT( Offset == inc_count, "reset(rf_reset_bodies) did not reset functor" ); 
-#endif
+        ASSERT( Offset == inc_count, "reset(rf_reset_bodies) did not reset functor" );
     }
 }
 
 template< typename InputType, typename OutputType >
 void run_buffered_levels( int c ) {
-    #if __TBB_LAMBDAS_PRESENT
+    #if __TBB_CPP11_LAMBDAS_PRESENT
     buffered_levels<InputType,OutputType>( c, []( InputType i ) -> OutputType { return harness_graph_executor<InputType, OutputType>::func(i); } );
     #endif
     buffered_levels<InputType,OutputType>( c, &harness_graph_executor<InputType, OutputType>::func );
@@ -236,108 +251,107 @@ void run_buffered_levels( int c ) {
 
 
 //! Performs test on executable nodes with limited concurrency
-/** Theses tests check:
+/** These tests check:
     1) that the nodes will accepts puts up to the concurrency limit,
-    2) the nodes do not exceed the concurrency limit even when run with more threads (this is checked in the harness_graph_executor), 
+    2) the nodes do not exceed the concurrency limit even when run with more threads (this is checked in the harness_graph_executor),
     3) the nodes will receive puts from multiple successors simultaneously,
     and 4) the nodes will send to multiple predecessors.
     There is no checking of the contents of the messages for corruption.
 */
-     
+
 template< typename InputType, typename OutputType, typename Body >
 void concurrency_levels( size_t concurrency, Body body ) {
 
-   for ( size_t lc = 1; lc <= concurrency; ++lc ) { 
-   tbb::flow::graph g;
+   for ( size_t lc = 1; lc <= concurrency; ++lc ) {
+       tbb::flow::graph g;
 
-   // Set the execute_counter back to zero in the harness
-   harness_graph_executor<InputType, OutputType>::execute_count = 0;
-   // Set the number of current executors to zero.
-   harness_graph_executor<InputType, OutputType>::current_executors = 0;
-   // Set the max allowed executors to lc. There is a check in the functor to make sure this is never exceeded.
-   harness_graph_executor<InputType, OutputType>::max_executors = lc;
+       // Set the execute_counter back to zero in the harness
+       harness_graph_executor<InputType, OutputType>::execute_count = 0;
+       // Set the number of current executors to zero.
+       harness_graph_executor<InputType, OutputType>::current_executors = 0;
+       // Set the max allowed executors to lc. There is a check in the functor to make sure this is never exceeded.
+       harness_graph_executor<InputType, OutputType>::max_executors = lc;
 
-   typedef tbb::flow::function_node< InputType, OutputType, tbb::flow::rejecting > fnode_type;
-   fnode_type exe_node( g, lc, body );
+       typedef tbb::flow::function_node< InputType, OutputType, tbb::flow::rejecting > fnode_type;
+       fnode_type exe_node( g, lc, body );
 
-   for (size_t num_receivers = 1; num_receivers <= MAX_NODES; ++num_receivers ) {
+       for (size_t num_receivers = 1; num_receivers <= MAX_NODES; ++num_receivers ) {
 
-        harness_counting_receiver<OutputType> *receivers = new harness_counting_receiver<OutputType>[num_receivers];
+            std::vector< harness_counting_receiver<OutputType> > receivers(num_receivers, harness_counting_receiver<OutputType>(g));
 
-#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
-        ASSERT(exe_node.successor_count() == 0, NULL);
-        ASSERT(exe_node.predecessor_count() == 0, NULL);
+#if TBB_DEPRECATED_FLOW_NODE_EXTRACTION
+            ASSERT(exe_node.successor_count() == 0, NULL);
+            ASSERT(exe_node.predecessor_count() == 0, NULL);
 #endif
 
-        for (size_t r = 0; r < num_receivers; ++r ) {
-            tbb::flow::make_edge( exe_node, receivers[r] );
-        }
-#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
-        ASSERT(exe_node.successor_count() == num_receivers, NULL);
-        typename fnode_type::successor_vector_type my_succs;
-        exe_node.copy_successors(my_succs);
-        ASSERT(my_succs.size() == num_receivers, NULL);
-        typename fnode_type::predecessor_vector_type my_preds;
-        exe_node.copy_predecessors(my_preds);
-        ASSERT(my_preds.size() == 0, NULL);
-#endif
-
-        harness_counting_sender<InputType> *senders = NULL;
-
-        for (size_t num_senders = 1; num_senders <= MAX_NODES; ++num_senders ) {
-            senders = new harness_counting_sender<InputType>[num_senders];
-            {
-                // Exclusively lock m to prevent exe_node from finishing
-                tbb::spin_rw_mutex::scoped_lock l( harness_graph_executor<InputType, OutputType>::template mutex_holder<tbb::spin_rw_mutex>::mutex );
-
-                // put to lc level, it will accept and then block at m
-                for ( size_t c = 0 ; c < lc ; ++c ) {
-                    ASSERT( exe_node.try_put( InputType() ) == true, NULL );
-                }
-                // it only accepts to lc level
-                ASSERT( exe_node.try_put( InputType() ) == false, NULL );
-
-                for (size_t s = 0; s < num_senders; ++s ) {
-                   // register a sender
-                   senders[s].my_limit = N;
-                   exe_node.register_predecessor( senders[s] );
-                }
-
-            } // release lock at end of scope, setting the exe node free to continue
-            // wait for graph to settle down
-            g.wait_for_all();
-
-            // confirm that each sender was requested from N times 
-            for (size_t s = 0; s < num_senders; ++s ) {
-                size_t n = senders[s].my_received;
-                ASSERT( n == N, NULL ); 
-                ASSERT( senders[s].my_receiver == &exe_node, NULL );
-            }
-            // confirm that each receivers got N * num_senders + the initial lc puts
             for (size_t r = 0; r < num_receivers; ++r ) {
-                size_t n = receivers[r].my_count;
-                ASSERT( n == num_senders*N+lc, NULL );
-                receivers[r].my_count = 0;
+                tbb::flow::make_edge( exe_node, receivers[r] );
             }
-            delete [] senders;
-        }
-        for (size_t r = 0; r < num_receivers; ++r ) {
-            tbb::flow::remove_edge( exe_node, receivers[r] );
-        }
-        ASSERT( exe_node.try_put( InputType() ) == true, NULL );
-        g.wait_for_all();
-        for (size_t r = 0; r < num_receivers; ++r ) {
-            ASSERT( int(receivers[r].my_count) == 0, NULL );
-        }
-        delete [] receivers;
-    }
+#if TBB_DEPRECATED_FLOW_NODE_EXTRACTION
+            ASSERT(exe_node.successor_count() == num_receivers, NULL);
+            typename fnode_type::successor_list_type my_succs;
+            exe_node.copy_successors(my_succs);
+            ASSERT(my_succs.size() == num_receivers, NULL);
+            typename fnode_type::predecessor_list_type my_preds;
+            exe_node.copy_predecessors(my_preds);
+            ASSERT(my_preds.size() == 0, NULL);
+#endif
 
+            harness_counting_sender<InputType> *senders = NULL;
+
+            for (size_t num_senders = 1; num_senders <= MAX_NODES; ++num_senders ) {
+                senders = new harness_counting_sender<InputType>[num_senders];
+                {
+                    // Exclusively lock m to prevent exe_node from finishing
+                    tbb::spin_rw_mutex::scoped_lock l( harness_graph_executor<InputType, OutputType>::template mutex_holder<tbb::spin_rw_mutex>::mutex );
+
+                    // put to lc level, it will accept and then block at m
+                    for ( size_t c = 0 ; c < lc ; ++c ) {
+                        ASSERT( exe_node.try_put( InputType() ) == true, NULL );
+                    }
+                    // it only accepts to lc level
+                    ASSERT( exe_node.try_put( InputType() ) == false, NULL );
+
+                    for (size_t s = 0; s < num_senders; ++s ) {
+                       // register a sender
+                       senders[s].my_limit = N;
+                       exe_node.register_predecessor( senders[s] );
+                    }
+
+                } // release lock at end of scope, setting the exe node free to continue
+                // wait for graph to settle down
+                g.wait_for_all();
+
+                // confirm that each sender was requested from N times
+                for (size_t s = 0; s < num_senders; ++s ) {
+                    size_t n = senders[s].my_received;
+                    ASSERT( n == N, NULL );
+                    ASSERT( senders[s].my_receiver == &exe_node, NULL );
+                }
+                // confirm that each receivers got N * num_senders + the initial lc puts
+                for (size_t r = 0; r < num_receivers; ++r ) {
+                    size_t n = receivers[r].my_count;
+                    ASSERT( n == num_senders*N+lc, NULL );
+                    receivers[r].my_count = 0;
+                }
+                delete [] senders;
+            }
+            for (size_t r = 0; r < num_receivers; ++r ) {
+                tbb::flow::remove_edge( exe_node, receivers[r] );
+            }
+            ASSERT( exe_node.try_put( InputType() ) == true, NULL );
+            g.wait_for_all();
+            for (size_t r = 0; r < num_receivers; ++r ) {
+                ASSERT( int(receivers[r].my_count) == 0, NULL );
+            }
+        }
     }
 }
 
+
 template< typename InputType, typename OutputType >
 void run_concurrency_levels( int c ) {
-    #if __TBB_LAMBDAS_PRESENT
+    #if __TBB_CPP11_LAMBDAS_PRESENT
     concurrency_levels<InputType,OutputType>( c, []( InputType i ) -> OutputType { return harness_graph_executor<InputType, OutputType>::template tfunc<tbb::spin_rw_mutex>(i); } );
     #endif
     concurrency_levels<InputType,OutputType>( c, &harness_graph_executor<InputType, OutputType>::template tfunc<tbb::spin_rw_mutex> );
@@ -345,7 +359,7 @@ void run_concurrency_levels( int c ) {
 }
 
 
-struct empty_no_assign { 
+struct empty_no_assign {
    empty_no_assign() {}
    empty_no_assign( int ) {}
    operator int() { return 0; }
@@ -368,7 +382,7 @@ struct parallel_puts : private NoAssign {
 };
 
 //! Performs test on executable nodes with unlimited concurrency
-/** Theses tests check:
+/** These tests check:
     1) that the nodes will accept all puts
     2) the nodes will receive puts from multiple predecessors simultaneously,
     and 3) the nodes will send to multiple successors.
@@ -384,7 +398,7 @@ void unlimited_concurrency( Body body ) {
 
         for (size_t num_receivers = 1; num_receivers <= MAX_NODES; ++num_receivers ) {
 
-            harness_counting_receiver<OutputType> *receivers = new harness_counting_receiver<OutputType>[num_receivers];
+            std::vector< harness_counting_receiver<OutputType> > receivers(num_receivers, harness_counting_receiver<OutputType>(g));
             harness_graph_executor<InputType, OutputType>::execute_count = 0;
 
             for (size_t r = 0; r < num_receivers; ++r ) {
@@ -392,24 +406,27 @@ void unlimited_concurrency( Body body ) {
             }
 
             NativeParallelFor( p, parallel_puts<InputType>(exe_node) );
-            g.wait_for_all(); 
+            g.wait_for_all();
 
             // 2) the nodes will receive puts from multiple predecessors simultaneously,
             size_t ec = harness_graph_executor<InputType, OutputType>::execute_count;
-            ASSERT( (int)ec == p*N, NULL ); 
+            ASSERT( (int)ec == p*N, NULL );
             for (size_t r = 0; r < num_receivers; ++r ) {
                 size_t c = receivers[r].my_count;
                 // 3) the nodes will send to multiple successors.
                 ASSERT( (int)c == p*N, NULL );
             }
+            for (size_t r = 0; r < num_receivers; ++r ) {
+                tbb::flow::remove_edge( exe_node, receivers[r] );
+            }
+            }
         }
     }
-}
 
 template< typename InputType, typename OutputType >
 void run_unlimited_concurrency() {
     harness_graph_executor<InputType, OutputType>::max_executors = 0;
-    #if __TBB_LAMBDAS_PRESENT
+    #if __TBB_CPP11_LAMBDAS_PRESENT
     unlimited_concurrency<InputType,OutputType>( []( InputType i ) -> OutputType { return harness_graph_executor<InputType, OutputType>::func(i); } );
     #endif
     unlimited_concurrency<InputType,OutputType>( &harness_graph_executor<InputType, OutputType>::func );
@@ -430,10 +447,10 @@ void test_function_node_with_continue_msg_as_input() {
 
     tbb::flow::function_node<tbb::flow::continue_msg, int, tbb::flow::rejecting> FN1( g, tbb::flow::serial, continue_msg_to_int(42));
     tbb::flow::function_node<tbb::flow::continue_msg, int, tbb::flow::rejecting> FN2( g, tbb::flow::serial, continue_msg_to_int(43));
-    
+
     tbb::flow::make_edge( Start, FN1 );
     tbb::flow::make_edge( Start, FN2 );
-    
+
     Start.try_put( tbb::flow::continue_msg() );
     g.wait_for_all();
 }
@@ -453,14 +470,14 @@ void test_concurrency(int num_threads) {
     test_function_node_with_continue_msg_as_input();
 }
 
-#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+#if TBB_DEPRECATED_FLOW_NODE_EXTRACTION
 struct add_to_counter {
     int* counter;
     add_to_counter(int& var):counter(&var){}
     int operator()(int i){*counter+=1; return i + 1;}
 };
 
-template<tbb::flow::graph_buffer_policy FTYPE>
+template<typename FTYPE>
 void test_extract() {
     int my_count = 0;
     int cm;
@@ -478,13 +495,13 @@ void test_extract() {
         ASSERT(b1.predecessor_count() == 0 && b1.successor_count() == 1, "b1 has incorrect counts");
         ASSERT(f0.predecessor_count() == 2 && f0.successor_count() == 1, "f0 has incorrect counts");
         ASSERT(q0.predecessor_count() == 1 && q0.successor_count() == 0, "q0 has incorrect counts");
-    
+
         /* b0         */
         /*   \        */
         /*    f0 - q0 */
         /*   /        */
         /* b1         */
-    
+
         b0.try_put(1);
         g.wait_for_all();
         ASSERT(my_count == 1, "function_node didn't fire");
@@ -493,15 +510,15 @@ void test_extract() {
         g.wait_for_all();
         ASSERT(my_count == 2, "function_node didn't fire");
         ASSERT(q0.try_get(cm), "function_node didn't forward");
-    
+
         b0.extract();
-    
+
         /* b0         */
         /*            */
         /*    f0 - q0 */
         /*   /        */
         /* b1         */
-    
+
         ASSERT(b0.predecessor_count() == 0 && b0.successor_count() == 0, "b0 has incorrect counts");
         ASSERT(b1.predecessor_count() == 0 && b1.successor_count() == 1, "b1 has incorrect counts");
         ASSERT(f0.predecessor_count() == 1 && f0.successor_count() == 1, "f0 has incorrect counts");
@@ -514,15 +531,15 @@ void test_extract() {
         g.wait_for_all();
         ASSERT(my_count == 3, "function_node didn't fire though it has only one predecessor");
         ASSERT(q0.try_get(cm), "function_node didn't forward second time");
-    
+
         f0.extract();
-    
+
         /* b0         */
         /*            */
         /*    f0   q0 */
         /*            */
         /* b1         */
-    
+
         ASSERT(b0.predecessor_count() == 0 && b0.successor_count() == 0, "b0 has incorrect counts");
         ASSERT(b1.predecessor_count() == 0 && b1.successor_count() == 0, "b1 has incorrect counts");
         ASSERT(f0.predecessor_count() == 0 && f0.successor_count() == 0, "f0 has incorrect counts");
@@ -535,24 +552,24 @@ void test_extract() {
         ASSERT(my_count == 3, "function_node didn't fire though it has only one predecessor");
         ASSERT(!q0.try_get(cm), "function_node forwarded though it shouldn't");
         make_edge(b0, f0);
-    
+
         /* b0         */
         /*   \        */
         /*    f0   q0 */
         /*            */
         /* b1         */
-    
+
         ASSERT(b0.predecessor_count() == 0 && b0.successor_count() == 1, "b0 has incorrect counts");
         ASSERT(b1.predecessor_count() == 0 && b1.successor_count() == 0, "b1 has incorrect counts");
         ASSERT(f0.predecessor_count() == 1 && f0.successor_count() == 0, "f0 has incorrect counts");
         ASSERT(q0.predecessor_count() == 0 && q0.successor_count() == 0, "q0 has incorrect counts");
-    
+
         b0.try_put(int());
         g.wait_for_all();
-    
+
         ASSERT(my_count == 4, "function_node didn't fire though it has only one predecessor");
         ASSERT(!q0.try_get(cm), "function_node forwarded though it shouldn't");
-    
+
         tbb::flow::make_edge(b1, f0);
         tbb::flow::make_edge(f0, q0);
         my_count = 0;
@@ -560,7 +577,7 @@ void test_extract() {
 }
 #endif
 
-int TestMain() { 
+int TestMain() {
     if( MinThread<1 ) {
         REPORT("number of threads must be positive\n");
         exit(1);
@@ -568,8 +585,8 @@ int TestMain() {
     for( int p=MinThread; p<=MaxThread; ++p ) {
        test_concurrency(p);
    }
-
-#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+   lightweight_testing::test<tbb::flow::function_node>(10);
+#if TBB_DEPRECATED_FLOW_NODE_EXTRACTION
     test_extract<tbb::flow::rejecting>();
     test_extract<tbb::flow::queueing>();
 #endif
