@@ -1,21 +1,17 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
 #ifdef __cplusplus
@@ -39,10 +35,19 @@ const int ExpectedResultHugePages = TBBMALLOC_OK;
 const int ExpectedResultHugePages = TBBMALLOC_NO_EFFECT;
 #endif
 
+/* bool type definition for C */
+#if (defined(_MSC_VER) && _MSC_VER < 1800) || __sun || __SUNPRO_CC
+typedef int bool;
+#define false 0
+#define true 1
+#else
+#include <stdbool.h>
+#endif
+
 #if __TBB_SOURCE_DIRECTLY_INCLUDED
 #include "../tbbmalloc/tbbmalloc_internal_api.h"
 #else
-#define __TBB_mallocProcessShutdownNotification()
+#define __TBB_mallocProcessShutdownNotification(bool)
 #endif
 
 /* test that it's possible to call allocation function from atexit
@@ -51,7 +56,7 @@ static void MyExit(void) {
     void *p = scalable_malloc(32);
     assert(p);
     scalable_free(p);
-    __TBB_mallocProcessShutdownNotification();
+    __TBB_mallocProcessShutdownNotification(false);
 }
 
 int main(void) {
@@ -117,7 +122,7 @@ int main(void) {
     res = scalable_allocation_command(TBBMALLOC_CLEAN_THREAD_BUFFERS,
                                       (void*)(intptr_t)1);
     assert(res == TBBMALLOC_INVALID_PARAM);
-    __TBB_mallocProcessShutdownNotification();
+    __TBB_mallocProcessShutdownNotification(false);
     printf("done\n");
     return 0;
 }
