@@ -1,24 +1,25 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
 #include "harness.h"
+
+#if __TBB_CPF_BUILD
+#define TBB_DEPRECATED_FLOW_NODE_EXTRACTION 1
+#endif
+
 #include "tbb/flow_graph.h"
 #include "tbb/task_scheduler_init.h"
 #include "tbb/tick_count.h"
@@ -73,11 +74,11 @@ struct touches {
     int my_num_threads;
 
     touches( int num_threads ) : my_num_threads(num_threads) {
-        my_last_touch = new T[my_num_threads]; 
-        my_touches = new bool* [my_num_threads]; 
+        my_last_touch = new T[my_num_threads];
+        my_touches = new bool* [my_num_threads];
         for ( int p = 0; p < my_num_threads; ++p) {
             my_last_touch[p] = T(-1);
-            my_touches[p] = new bool[N]; 
+            my_touches[p] = new bool[N];
             for ( int n = 0; n < N; ++n)
                 my_touches[p][n] = false;
         }
@@ -100,8 +101,8 @@ struct touches {
             printf("Error: value seen in wrong order by local thread\n");
             return false;
         }
-        my_last_touch[tid] = v; 
-        my_touches[tid][v] = true; 
+        my_last_touch[tid] = v;
+        my_touches[tid][v] = true;
         return true;
     }
 
@@ -109,7 +110,7 @@ struct touches {
         bool *all_touches = new bool[N];
         for ( int n = 0; n < N; ++n)
             all_touches[n] = false;
-     
+
         for ( int p = 0; p < my_num_threads; ++p) {
             for ( int n = 0; n < N; ++n) {
                 if ( my_touches[p][n] == true ) {
@@ -123,7 +124,7 @@ struct touches {
                 printf("No touch at %d, my_num_threads = %d\n", n, my_num_threads);
             //ASSERT( all_touches[n] == true, "value not seen by any thread\n" );
         }
-        delete all_touches;
+        delete [] all_touches;
         return true;
     }
 
@@ -141,7 +142,7 @@ struct parallel_gets : NoAssign {
     void operator()(int tid) const {
         for (int j = tid; j < N; j+=my_num_threads) {
             T v;
-            spin_try_get( my_q, v );    
+            spin_try_get( my_q, v );
             my_touches.check( tid, v );
         }
     }
@@ -154,10 +155,10 @@ struct parallel_put_get : NoAssign {
     tbb::flow::sequencer_node<T> &my_s1;
     tbb::flow::sequencer_node<T> &my_s2;
     int my_num_threads;
-    tbb::atomic< int > &my_counter;    
-    touches<T> &my_touches; 
+    tbb::atomic< int > &my_counter;
+    touches<T> &my_touches;
 
-    parallel_put_get( tbb::flow::sequencer_node<T> &s1, tbb::flow::sequencer_node<T> &s2, int num_threads, 
+    parallel_put_get( tbb::flow::sequencer_node<T> &s1, tbb::flow::sequencer_node<T> &s2, int num_threads,
                       tbb::atomic<int> &counter, touches<T> &t ) : my_s1(s1), my_s2(s2), my_num_threads(num_threads), my_counter(counter), my_touches(t) {}
 
     void operator()(int tid) const {
@@ -172,7 +173,7 @@ struct parallel_put_get : NoAssign {
 
             for (int i = i_start; i < i_end; ++i) {
                 T v;
-                spin_try_get( my_s2, v );    
+                spin_try_get( my_s2, v );
                 my_touches.check( tid, v );
             }
         }
@@ -180,7 +181,7 @@ struct parallel_put_get : NoAssign {
 
 };
 
-// 
+//
 // Tests
 //
 // multiple parallel senders, multiple receivers, properly sequenced (relative to receiver) at output
@@ -244,7 +245,7 @@ int test_parallel(int num_threads) {
 }
 
 
-// 
+//
 // Tests
 //
 // No predecessors can be registered
@@ -388,14 +389,14 @@ int test_serial() {
     return 0;
 }
 
-int TestMain() { 
+int TestMain() {
     tbb::tick_count start = tbb::tick_count::now(), stop;
     for (int p = 2; p <= 4; ++p) {
         tbb::task_scheduler_init init(p);
         test_serial<int>();
         test_parallel<int>(p);
-    } 
-#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+    }
+#if TBB_DEPRECATED_FLOW_NODE_EXTRACTION
     test_buffer_extract<tbb::flow::sequencer_node<int> >().run_tests();
 #endif
     stop = tbb::tick_count::now();
